package cn.com.qmth.examcloud.core.basic.api.controller;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.PropertiesUtil;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelError;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.base.constants.PropKeys;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Org;
import cn.com.qmth.examcloud.core.basic.service.bean.OrgDto;
import cn.com.qmth.examcloud.core.basic.service.impl.ExportService;
import cn.com.qmth.examcloud.core.basic.service.impl.OrgService;
import io.swagger.annotations.ApiOperation;

/**
 * 机构服务API Created by songyue on 17/1/14.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/org")
public class OrgController extends ControllerSupport {

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	OrgService orgService;

	@Autowired
	private JdbcTemplate jdbcTemplate;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param id
	 * @return
	 */
	@ApiOperation(value = "按ID查询机构", notes = "ID查询")
	@GetMapping("/{id}")
	public Org getOrg(@PathVariable Long id) {
		Org org = orgService.findOne(id);
		return org;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param parentId
	 * @return
	 */
	@ApiOperation(value = "查询下属机构不带分页", notes = "不分页")
	@GetMapping("/sub/{parentId}")
	public List<Org> getSubOrgList(@PathVariable Long parentId) {
		List<Org> orgList = orgRepo.findByParentIdAndEnable(parentId, true);
		return orgList;
	}

	/**
	 * 修改请通知作者
	 *
	 * @author WANGWEI
	 * @param curPage
	 * @param pageSize
	 * @param rootOrgId
	 * @param code
	 * @param name
	 * @return
	 */
	@ApiOperation(value = "模糊查询顶级机构的子机构")
	@GetMapping("/getSubOrgList/{curPage}/{pageSize}")
	public Page<Org> getSubOrgListOfRootOrg(@PathVariable Integer curPage,
			@PathVariable Integer pageSize, @RequestParam Long rootOrgId, @RequestParam String code,
			@RequestParam String name) {

		User accessUser = getAccessUser();
		if (!isSuperAdmin()) {
			if (!accessUser.getRootOrgId().equals(rootOrgId)) {
				throw new StatusException("B-140001", "非法访问");
			}
		}

		Pageable pageable = new PageRequest(curPage - 1, pageSize, Sort.Direction.DESC,
				"updateTime");

		Org criteria = new Org();
		criteria.setParentId(rootOrgId);
		criteria.setCode(code);
		criteria.setName(name);

		Specification<Org> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			if (StringUtils.isNotEmpty(criteria.getName())) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(criteria.getName())));
			}
			if (StringUtils.isNotEmpty(criteria.getCode())) {
				predicates.add(cb.like(root.get("code"), toSqlSearchPattern(criteria.getCode())));
			}
			predicates.add(cb.equal(root.get("parentId"), criteria.getParentId()));
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		Page<Org> ret = orgRepo.findAll(specification, pageable);
		return ret;
	}

	/**
	 * 修改请通知作者
	 *
	 * @author WANGWEI
	 * @return
	 */
	@ApiOperation(value = "查询所有顶级机构")
	@GetMapping("/getAllRootOrgList")
	public List<Org> getAllRootOrgList() {

		Specification<Org> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.isNull(root.get("parentId")));
			predicates.add(cb.equal(root.get("enable"), true));
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		List<Org> ret = orgRepo.findAll(specification);
		return ret;
	}

	/**
	 * 修改请通知作者
	 *
	 * @author WANGWEI
	 * @param curPage
	 * @param pageSize
	 * @param rootOrgId
	 * @param code
	 * @param name
	 * @return
	 */
	@ApiOperation(value = "模糊查询顶级机构")
	@GetMapping("/getRootOrgList/{curPage}/{pageSize}")
	public Page<Org> getRootOrgList(@PathVariable Integer curPage, @PathVariable Integer pageSize,
			@RequestParam String code, @RequestParam String name) {

		Pageable pageable = new PageRequest(curPage - 1, pageSize, Sort.Direction.DESC,
				"updateTime");

		Org criteria = new Org();
		criteria.setCode(code);
		criteria.setName(name);

		Specification<Org> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			if (StringUtils.isNotEmpty(criteria.getName())) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(criteria.getName())));
			}
			if (StringUtils.isNotEmpty(criteria.getCode())) {
				predicates.add(cb.like(root.get("code"), toSqlSearchPattern(criteria.getCode())));
			}
			predicates.add(cb.isNull(root.get("parentId")));
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		Page<Org> ret = orgRepo.findAll(specification, pageable);
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param org
	 * @param request
	 * @return
	 */
	@ApiOperation(value = "新增顶级机构", notes = "新增")
	@PostMapping("addRootOrg")
	public Org addRootOrg(@RequestBody Org org) {

		if (!isSuperAdmin()) {
			throw new StatusException("B-140001", "非法访问");
		}

		org.setParentId(null);
		Org saved = orgService.save(org);
		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param org
	 * @return
	 */
	@ApiOperation(value = "更新顶级机构", notes = "更新")
	@PutMapping("updateRootOrg")
	public Org updateRootOrg(@RequestBody Org org) {
		trim(org);
		if (!isSuperAdmin()) {
			throw new StatusException("B-140001", "非法访问");
		}

		org.setParentId(null);
		org.setRootId(org.getId());
		Org saved = orgService.save(org);
		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param org
	 * @return
	 */
	@ApiOperation(value = "新增子机构", notes = "新增")
	@PostMapping("addSubOrg")
	public Org addSubOrg(@RequestBody Org org) {
		trim(org);
		User accessUser = getAccessUser();

		Org subOrg = orgRepo.findByRootIdAndCode(accessUser.getRootOrgId(), org.getCode());
		if (null != subOrg) {
			throw new StatusException("B-140001", "机构代码已存在");
		}
		org.setParentId(accessUser.getRootOrgId());
		org.setRootId(accessUser.getRootOrgId());
		Org saved = orgService.save(org);

		// orgService.createLearnerCenterUser(saved);
		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param org
	 * @param request
	 * @return
	 */
	@ApiOperation(value = "更新子机构", notes = "更新")
	@PutMapping("updateSubOrg")
	public Org updateSubOrg(@RequestBody Org org) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();

		org.setParentId(accessUser.getRootOrgId());
		org.setRootId(accessUser.getRootOrgId());
		Org saved = orgService.save(org);
		return saved;
	}

	@ApiOperation(value = "按ID删除机构", notes = "删除")
	@DeleteMapping("/{id}")
	public void deleteSchool(@PathVariable Long id) {
		orgRepo.delete(id);
	}

	@ApiOperation(value = "按父ID导入子机构", notes = "导入子机构")
	@PostMapping("/import")
	public List<ExcelError> importLearnCenter(HttpServletRequest request,
			@RequestParam CommonsMultipartFile file) throws Exception {
		User accessUser = getAccessUser();

		List<ExcelError> excelErrors = orgService.importLearnCenter(accessUser.getRootOrgId(),
				file.getInputStream());
		return excelErrors;
	}

	@ApiOperation(value = "下载导入模板", notes = "下载导入模板")
	@GetMapping("/download")
	public void importFileTemplate(HttpServletResponse response) {
		List<OrgDto> list = new ArrayList<OrgDto>();
		ExportService.exportEXCEL("学习中心导入模板", OrgDto.class, list, response);
	}

	@ApiOperation(value = "启用机构", notes = "启用")
	@PutMapping("/enable/{ids}")
	public void enableSchool(@PathVariable String ids) {
		List<Long> orgIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long orgId : orgIds) {
			orgService.enableSchool(orgId, true);
		}
	}

	@ApiOperation(value = "禁用机构", notes = "禁用")
	@PutMapping("/disable/{ids}")
	public void disableSchool(@PathVariable String ids) {
		List<Long> orgIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long orgId : orgIds) {
			orgService.enableSchool(orgId, false);
		}
	}

	@ApiOperation(value = "查询logo")
	@GetMapping("/logo")
	public void getLogo(@RequestParam("domain") String domain, HttpServletResponse response)
			throws IOException {
		Org org = orgRepo.findRootOrg(domain);
		String logo = org.getLogo();
		IOUtils.copy(new FileInputStream(logo), response.getOutputStream());
		response.flushBuffer();
	}

	@ApiOperation(value = "查询顶级机构")
	@GetMapping("/getRootOrgByCode")
	public Org getRootOrgByCode(@RequestParam("code") String code) throws IOException {
		Org org = orgRepo.findRootOrg(code);
		return org;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param id
	 * @param request
	 * @param file
	 * @throws IOException
	 */
	@ApiOperation(value = "导入logo", notes = "导入logo")
	@PostMapping("/importLogo/{id}")
	public void importLogo(@PathVariable Long id, HttpServletRequest request,
			@RequestParam CommonsMultipartFile file) throws IOException {
		String schoolLogoPath = PropertiesUtil.getString(PropKeys.SCHOOL_LOGO_PATH);
		if (StringUtils.isBlank(schoolLogoPath)) {
			FileUtils.forceMkdir(new File(schoolLogoPath));
		}
		FileItem fileItem = file.getFileItem();
		String name = fileItem.getName();
		String filePath = null;
		if (name.endsWith(".jpg")) {
			filePath = schoolLogoPath + "/" + id + ".jpg";
		} else if (name.endsWith(".gif")) {
			filePath = schoolLogoPath + "/" + id + ".gif";
		} else if (name.endsWith(".png")) {
			filePath = schoolLogoPath + "/" + id + ".png";
		} else {
			throw new StatusException("B-101001", "文件格式错误");
		}

		FileUtils.copyInputStreamToFile(file.getInputStream(), new File(filePath));
		jdbcTemplate.update("update ecs_core_org t set  t.logo=? where t.id=?", filePath, id);
	}

	/**
	 * 修改请通知作者
	 *
	 * @author WANGWEI
	 * @param parentId
	 * @param orgName
	 * @return
	 */
	@ApiOperation(value = "按机构名称模糊查询子机构列表")
	@PostMapping("getSubOrgListByNameLike/{parentId}")
	public List<Org> getSubOrgListByNameLike(@PathVariable Long parentId,
			@RequestParam String orgName) {
		List<Org> list = null;
		if (StringUtils.isBlank(orgName)) {
			list = Lists.newArrayList();
			return list;
		}
		orgName = "%" + orgName + "%";
		list = orgRepo.findByParentIdAndNameLikeAndEnable(parentId, orgName, true);
		return list;
	}

}
